package com.engine.attendance.attendanceanalysis.cmd.workovertime;

import com.engine.attendance.enums.AccountingUnitEnum;
import com.engine.attendance.enums.CheckBoxEnum;
import com.engine.attendance.enums.DateTypeEnum;
import com.engine.attendance.enums.DelayTypeEnum;
import com.engine.common.biz.AbstractCommonCommand;
import com.engine.common.entity.BizLogContext;
import com.engine.common.util.DateUtil;
import com.engine.common.util.DbTools;
import com.engine.common.util.Utils;
import com.engine.core.interceptor.CommandContext;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.general.TimeUtil;
import weaver.general.Util;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 根据相关设置控制加班时长，并入库
 */
@Slf4j
public class SaveWorkOverTimeCmd extends AbstractCommonCommand<Map<String,Object>> {
    public SaveWorkOverTimeCmd(Map<String, Object> params) {
        this.params = params;
    }
    @Override
    public BizLogContext getLogContext() {
        return null;
    }

    @Override
    public Map<String, Object> execute(CommandContext commandContext) {

        //加班项目
        List<Map<String, Object>> workTimeBeLateItems = (List<Map<String,Object>>)params.get("workTimeBeLateItems");
        //加班时长
        double jbsc = Double.valueOf(params.get("jbsc").toString());
        //班次
        Map<String, Object> scheduleMap = (Map<String, Object>)params.get("scheduleMap");

        Map<String,Object> workOverTimeItems = Maps.newHashMap();


        //超出限制时长的处理方式
        String ccclfs = Util.null2String(workTimeBeLateItems.get(0).get("ccclfs"));
        //工作日加班最大小时数
        String rzdjbxss = Util.null2String(workTimeBeLateItems.get(0).get("rzdjbxss"));
        //每周最大加班小时数
        String yzdjbxss = Util.null2String(workTimeBeLateItems.get(0).get("yzdjbxss"));
        //每月最大加班小时数
        String zzdjbxss = Util.null2String(workTimeBeLateItems.get(0).get("zzdjbxss"));
        //加班时长自动转入假期余额
        String jbzdzjqye= Util.null2String(workTimeBeLateItems.get(0).get("jbzdzjqye"));
        //最小加班分钟数
        String jbqsfzs = Util.null2String(workTimeBeLateItems.get(0).get("jbqsfzs"));
        //超出最小时长后的单次累加分钟数
        String ccqszhdhsfzs = Util.null2String(workTimeBeLateItems.get(0).get("ccqszhdhsfzs"));
        //核算量
        double workOverTimeHsl = Double.valueOf(Util.null2String(workTimeBeLateItems.get(0).get("hsl")));
        //核算单位
        String workOverTimeHsdw = Util.null2String(workTimeBeLateItems.get(0).get("hsdw"));
        //分析日期
        String analysisDate = Util.null2String(params.get("analysisDate"));
        //实际开始时间
        String realityStartTime = Util.null2String(params.get("realityStartTime"));
        //实际结束时间
        String realityEndime = Util.null2String(params.get("realityEndime"));

        long diffdays = Long.valueOf(Util.null2String(params.get("diffdays")));


        String rqlx = Util.null2String(scheduleMap.get("rqlx"));

        List<Map<String, Object>> workOverTimeResults = null;

        String userId = Util.null2String(params.get("userId"));

        if (!"".equals(jbqsfzs)){
            if (jbsc < Integer.valueOf(jbqsfzs)){
                //小于最小加班分钟数不算加班
                workOverTimeItems.put("item",workTimeBeLateItems.get(0).get("key"));
                workOverTimeItems.put("itemduration","0");
                return workOverTimeItems;
            }else if (jbsc >= Integer.valueOf(jbqsfzs) && !"".equals(ccqszhdhsfzs)){

                jbsc = Double.valueOf(Utils.getItemdurationDown(Integer.valueOf(ccqszhdhsfzs), AccountingUnitEnum.MINUTES.getKey(),Double.valueOf(jbsc).intValue()-Integer.valueOf(jbqsfzs),AccountingUnitEnum.MINUTES)).intValue()+Integer.valueOf(jbqsfzs);
            }
        }
        if (jbsc <= 0 || "".equals(realityStartTime) || "".equals(realityEndime)){
            //加班时长为0
            workOverTimeItems.put("item",workTimeBeLateItems.get(0).get("key"));
            workOverTimeItems.put("itemduration","0");
            return workOverTimeItems;
        }


        //自动扣除
        if ("2".equals(ccclfs) && workOverTimeResults == null){
            String startDate = analysisDate.split("-")[0]+"-"+ analysisDate.split("-")[1]+"-01";
            String endDate = analysisDate.split("-")[0]+"-"+ analysisDate.split("-")[1]+"-"+diffdays;
            workOverTimeResults =getWorkOverTimeResults(startDate,endDate,userId);

        }
        if ("2".equals(ccclfs) && !"".equals(rzdjbxss) && DateTypeEnum.WORK_DAY.getKey().equals(rqlx)){
            //工作日加班最大数
            double dayTime = getWorkDayTime(workOverTimeResults,analysisDate) *60;
            double maxDayTime = Double.valueOf(rzdjbxss) *60;
            double time = jbsc+dayTime;
            if (time > maxDayTime){
                jbsc = maxDayTime-dayTime;
            }
        }
        if ("2".equals(ccclfs) && !"".equals(zzdjbxss)){
            //每周最大加班小时数
            double weekTime = getWeekTime(workOverTimeResults,analysisDate);
            double maxWeekTime = Double.valueOf(zzdjbxss) *60;
            double time = jbsc+weekTime;
            if (time > maxWeekTime){
                jbsc = maxWeekTime-weekTime;
            }
        }
        if ("2".equals(ccclfs) && !"".equals(yzdjbxss)){
            //每月最大加班数
            double monthTime = getMonthTime(workOverTimeResults);
            double maxMonthTime = Double.valueOf(yzdjbxss) *60;
            double time = jbsc+monthTime;
            if (time > maxMonthTime){
                jbsc = maxMonthTime-monthTime;
            }
        }
        log.info("最终加班时长: {}",jbsc);
        workOverTimeItems.put("item",workTimeBeLateItems.get(0).get("key"));
        double itemduration = Utils.getItemdurationDown(workOverTimeHsl,workOverTimeHsdw,Long.valueOf(Math.round(jbsc)).intValue(),AccountingUnitEnum.MINUTES);
        workOverTimeItems.put("itemduration",itemduration);

        /**
         * 入加班结果表
         */
        Map<String,Object> insertParam = Maps.newHashMap();
        insertParam.put("jbry",userId);
        insertParam.put("sjksrq",realityStartTime.split(" ")[0]);
        insertParam.put("sjkssj",realityStartTime.split(" ")[1]);
        insertParam.put("sjjsrq",realityEndime.split(" ")[0]);
        insertParam.put("sjjssj",realityEndime.split(" ")[1]);
        insertParam.put("sjjbsc",itemduration);
        insertParam.put("jbjgly","4");
        insertParam.put("jblx",workTimeBeLateItems.get(0).get("id"));
        insertParam.put("lyid",userId+"-"+ DateUtil.getCurrentDate());
        insertParam.put("zt","1");
        Utils.InsertFormTable("uf_jcl_kq_jbjg",insertParam,(Map<String,String>)params.get("formmodeIdMap"));

        /**
         * 入假期余额
         */
        //加班时长自动转入假期余额
        if (CheckBoxEnum.CHECKED.getKey().equals(jbzdzjqye)){
            Map<String,Object> insertHoliDayParam = Maps.newHashMap();
            insertHoliDayParam.put("ygid",userId);
            insertHoliDayParam.put("jqid",workTimeBeLateItems.get(0).get("zrdjb"));
            insertHoliDayParam.put("lyid",userId+"-"+DateUtil.getCurrentDate());
            //转入的假期额度生效日期
            String yesxrq = Util.null2String(workTimeBeLateItems.get(0).get("yesxrq"));
            if ("0".equals(yesxrq)){
                //一月后
                insertHoliDayParam.put("sxrq",DateUtil.lastMonth(analysisDate,DateUtil.yyyyMMdd));
            }else if ("1".equals(yesxrq)){
                //次月
                insertHoliDayParam.put("sxrq",DateUtil.lastMonth(analysisDate)+"-01");
            }else if ("2".equals(yesxrq)){
                //次日
                insertHoliDayParam.put("sxrq",DateUtil.AfterDay(analysisDate,1));
            }
            //额度可用的周期
            String yekyzq = Util.null2String(workTimeBeLateItems.get(0).get("yekyzq"));
            if ("0".equals(yekyzq)){
                //一个月
                insertHoliDayParam.put("jzrq",DateUtil.nextMonth(insertHoliDayParam.get("sxrq").toString(),1,DateUtil.yyyyMMdd));
            }else if ("1".equals(yekyzq)){
                //两个月
                insertHoliDayParam.put("jzrq",DateUtil.nextMonth(insertHoliDayParam.get("sxrq").toString(),2,DateUtil.yyyyMMdd));
            }else if ("2".equals(yekyzq)){
                //三个月
                insertHoliDayParam.put("jzrq",DateUtil.nextMonth(insertHoliDayParam.get("sxrq").toString(),3,DateUtil.yyyyMMdd));
            }else if ("3".equals(yekyzq)){
                //六个月
                insertHoliDayParam.put("jzrq",DateUtil.nextMonth(insertHoliDayParam.get("sxrq").toString(),6,DateUtil.yyyyMMdd));
            }else if ("4".equals(yekyzq)){
                //十二个月
                insertHoliDayParam.put("jzrq",DateUtil.nextMonth(insertHoliDayParam.get("sxrq").toString(),12,DateUtil.yyyyMMdd));
            }else if ("5".equals(yekyzq)){
                //当前季度
                insertHoliDayParam.put("jzrq",DateUtil.getCurrentQuarter(insertHoliDayParam.get("sxrq").toString()));
            }else if ("6".equals(yekyzq)){
                //当前半年
                insertHoliDayParam.put("jzrq",DateUtil.getCurrentHalfYear(insertHoliDayParam.get("sxrq").toString()));
            }else if ("7".equals(yekyzq)){
                //当前一年
                insertHoliDayParam.put("jzrq",insertHoliDayParam.get("sxrq").toString().split("-")[0]+"-12-31");
            }
            insertHoliDayParam.put("ktsc",itemduration);

            String zdyqsc = Util.null2String(workTimeBeLateItems.get(0).get("zdyqsc"));
            if (DelayTypeEnum.ONE_MONTH.getKey().equals(zdyqsc)){
                //一个月
                insertHoliDayParam.put("yqsxrq",DateUtil.nextMonth(insertHoliDayParam.get("jzrq").toString(),1,DateUtil.yyyyMMdd));
            }else if (DelayTypeEnum.TWO_MONTH.getKey().equals(zdyqsc)){
                //两个月
                insertHoliDayParam.put("yqsxrq",DateUtil.nextMonth(insertHoliDayParam.get("jzrq").toString(),2,DateUtil.yyyyMMdd));
            }else if (DelayTypeEnum.THREE_MONTH.getKey().equals(zdyqsc)){
                //三个月
                insertHoliDayParam.put("yqsxrq",DateUtil.nextMonth(insertHoliDayParam.get("jzrq").toString(),3,DateUtil.yyyyMMdd));
            }else if (DelayTypeEnum.HALF_YEAR.getKey().equals(zdyqsc)){
                //半年
                insertHoliDayParam.put("yqsxrq",DateUtil.nextMonth(insertHoliDayParam.get("jzrq").toString(),6,DateUtil.yyyyMMdd));
            }else if (DelayTypeEnum.ONE_YEAR.getKey().equals(zdyqsc)){
                //一年
                insertHoliDayParam.put("yqsxrq",DateUtil.nextMonth(insertHoliDayParam.get("jzrq").toString(),12,DateUtil.yyyyMMdd));
            }
            insertHoliDayParam.put("yxsc","0");
            insertHoliDayParam.put("wxsc","0");
            insertHoliDayParam.put("ztsc","0");
            Utils.InsertFormTable("uf_jcl_kq_jqye",insertHoliDayParam,(Map<String,String>)params.get("formmodeIdMap"));

        }
        return workOverTimeItems;
    }

    public List<Map<String, Object>> getWorkOverTimeResults(String startDate,String endDate,String userId){
        String sql = "select sjjbsc,sjksrq,sjjsrq from uf_jcl_kq_jbjg where jbry=? and sjksrq>? and sjjsrq<?";
        List<Map<String, Object>> dataList = DbTools.getSqlToList(sql,userId,startDate,endDate);
        return dataList;
    }
    /**
     * 获得工作日加班数
     * @param dataList
     * @return
     */
    public int getWorkDayTime(List<Map<String, Object>> dataList,String date){
        List<Map<String, Object>> list = dataList.stream().filter(e->date.equals(e.get("sjksrq"))).collect(Collectors.toList());
        int totalHour = list.stream().mapToInt(e->Integer.valueOf(e.get("sjjbsc").toString())).sum();
        return totalHour;
    }
    /**
     * 获得一周加班数
     * @param dataList
     * @return
     */
    public int getWeekTime(List<Map<String, Object>> dataList,String date){
        int day = TimeUtil.getDayOfWeek(date);
        if (day ==0){
            day = 7;
        }
        String startDate = DateUtil.beforeDay(date,day-1);
        String endDate = DateUtil.AfterDay(date,7-day);
        List<Map<String, Object>> list = dataList.stream().filter(e->{
            String sjksrq = Util.null2String(e.get("sjksrq"));
            if (DateUtil.getTime(sjksrq).compareTo(DateUtil.getTime(startDate)) >=0 &&
                    DateUtil.getTime(sjksrq).compareTo(DateUtil.getTime(endDate)) >=0){
                return true;
            }else {
                return false;
            }
        }).collect(Collectors.toList());
        int totalHour = list.stream().mapToInt(e->Integer.valueOf(e.get("sjjbsc").toString())).sum();
        return totalHour;
    }
    /**
     * 获得一个月加班数
     * @param dataList
     * @return
     */
    public int getMonthTime(List<Map<String, Object>> dataList){
        int totalHour = dataList.stream().mapToInt(e->Integer.valueOf(e.get("sjjbsc").toString())).sum();
        return totalHour;
    }
}
